@@ -21,12 +21,33 @@ double f_llik_0(int p, NumericVector par, void *ex) {
   }
 
   double loglik = 0;
-  for (person = 0; person < n; person++) {
-    double f = param->freq[person];
-    double w = param->weight[person];
-    double y = param->y[person];
-    double v = eta[person]*y + R::plogis(eta[person], 0, 1, 0, 1);
-    loglik += f*w*v;
+  if (param->link == "logit") {
+    for (person = 0; person < n; person++) {
+      double f = param->freq[person];
+      double w = param->weight[person];
+      double y = param->y[person];
+      double r = R::plogis(eta[person], 0, 1, 1, 0);
+      double v = y*eta[person] + log(1-r);
+      loglik += f*w*v;
+    }
+  } else if (param->link == "probit") {
+    for (person = 0; person < n; person++) {
+      double f = param->freq[person];
+      double w = param->weight[person];
+      double y = param->y[person];
+      double r = R::pnorm(eta[person], 0, 1, 1, 0);
+      double v = y*log(r/(1-r)) + log(1-r);
+      loglik += f*w*v;
+    }
+  } else if (param->link == "cloglog") {
+    for (person = 0; person < n; person++) {
+      double f = param->freq[person];
+      double w = param->weight[person];
+      double y = param->y[person];
+      double r = 1 - exp(-exp(eta[person]));
+      double v = y*log(r/(1-r)) + log(1-r);
+      loglik += f*w*v;
+    }
   }
 
   return loglik;
@@ -48,13 +69,42 @@ NumericVector f_score_0(int p, NumericVector par, void *ex) {
   }
 
   NumericVector score(p);
-  for (person = 0; person < n; person++) {
-    double f = param->freq[person];
-    double w = param->weight[person];
-    double v = param->y[person] - R::plogis(eta[person], 0, 1, 1, 0);
-    NumericVector z = param->z(person, _);
-    for (i=0; i<p; i++) {
-      score[i] += f*w*v*z[i];
+  if (param->link == "logit") {
+    for (person = 0; person < n; person++) {
+      double f = param->freq[person];
+      double w = param->weight[person];
+      double r = R::plogis(eta[person], 0, 1, 1, 0);
+      double v = param->y[person] - r;
+      NumericVector z = param->z(person, _);
+      for (i=0; i<p; i++) {
+        score[i] += f*w*v*z[i];
+      }
+    }
+  } else if (param->link == "probit") {
+    for (person = 0; person < n; person++) {
+      double f = param->freq[person];
+      double w = param->weight[person];
+      double r = R::pnorm(eta[person], 0, 1, 1, 0);
+      double phi = R::dnorm(eta[person], 0, 1, 0);
+      double d = phi/(r*(1-r));
+      double v = param->y[person] - r;
+      NumericVector z = param->z(person, _);
+      for (i=0; i<p; i++) {
+        score[i] += f*w*v*d*z[i];
+      }
+    }
+  } else if (param->link == "cloglog") {
+    for (person = 0; person < n; person++) {
+      double f = param->freq[person];
+      double w = param->weight[person];
+      double r = 1 - exp(-exp(eta[person]));
+      double phi = exp(eta[person] - exp(eta[person]));
+      double d = phi/(r*(1-r));
+      double v = param->y[person] - r;
+      NumericVector z = param->z(person, _);
+      for (i=0; i<p; i++) {
+        score[i] += f*w*v*d*z[i];
+      }
     }
   }
 
@@ -62,7 +112,7 @@ NumericVector f_score_0(int p, NumericVector par, void *ex) {
 }
 
 
-// observed information matrix
+// expected information matrix
 NumericMatrix f_info_0(int p, NumericVector par, void *ex) {
   logparams *param = (logparams *) ex;
   int n = param->n;
@@ -77,14 +127,44 @@ NumericMatrix f_info_0(int p, NumericVector par, void *ex) {
   }
 
   NumericMatrix imat(p,p);
-  for (person = 0; person < n; person++) {
-    double f = param->freq[person];
-    double w = param->weight[person];
-    double v = R::dlogis(eta[person], 0, 1, 0);
-    NumericVector z = param->z(person, _);
-    for (i=0; i<p; i++) {
-      for (j=0; j<=i; j++) {
-        imat(i,j) += f*w*v*z[i]*z[j];
+  if (param->link == "logit") {
+    for (person = 0; person < n; person++) {
+      double f = param->freq[person];
+      double w = param->weight[person];
+      double v = R::dlogis(eta[person], 0, 1, 0);
+      NumericVector z = param->z(person, _);
+      for (i=0; i<p; i++) {
+        for (j=0; j<=i; j++) {
+          imat(i,j) += f*w*v*z[i]*z[j];
+        }
+      }
+    }
+  } else if (param->link == "probit") {
+    for (person = 0; person < n; person++) {
+      double f = param->freq[person];
+      double w = param->weight[person];
+      double r = R::pnorm(eta[person], 0, 1, 1, 0);
+      double phi = R::dnorm(eta[person], 0, 1, 0);
+      double v = phi*phi/(r*(1-r));
+      NumericVector z = param->z(person, _);
+      for (i=0; i<p; i++) {
+        for (j=0; j<=i; j++) {
+          imat(i,j) += f*w*v*z[i]*z[j];
+        }
+      }
+    }
+  } else if (param->link == "cloglog") {
+    for (person = 0; person < n; person++) {
+      double f = param->freq[person];
+      double w = param->weight[person];
+      double r = 1 - exp(-exp(eta[person]));
+      double phi = exp(eta[person] - exp(eta[person]));
+      double v = phi*phi/(r*(1-r));
+      NumericVector z = param->z(person, _);
+      for (i=0; i<p; i++) {
+        for (j=0; j<=i; j++) {
+          imat(i,j) += f*w*v*z[i]*z[j];
+        }
       }
     }
   }
@@ -118,7 +198,7 @@ double f_pen_llik_0(int p, NumericVector par, void *ex) {
 }
 
 
-// Firth's bias-reducing penalized score vector
+// Firth's penalized score vector
 NumericVector f_pen_score_0(int p, NumericVector par, void *ex) {
   logparams *param = (logparams *) ex;
   int n = param->n;
@@ -132,51 +212,192 @@ NumericVector f_pen_score_0(int p, NumericVector par, void *ex) {
     }
   }
 
-  NumericVector pi(n);
-  for (person=0; person<n; person++) {
-    pi[person] = R::plogis(eta[person], 0, 1, 1, 0);
-  }
-
-  NumericMatrix xw2(p,n);  // X^T W^{1/2}
-  for (person=0; person<n; person++) {
-    double f = param->freq[person];
-    double w = param->weight[person];
-    double w2 = sqrt(f*w*pi[person]*(1-pi[person]));
-    for (i=0; i<p; i++) {
-      xw2(i,person) = param->z(person,i)*w2;
+  NumericVector pi(n), d(n), a(n), b(n);
+  if (param->link == "logit") {
+    for (person = 0; person < n; person++) {
+      double r = R::plogis(eta[person], 0, 1, 1, 0);
+      pi[person] = r;
+      d[person] = 1;
+      a[person] = r*(1-r);
+      b[person] = 1-2*r;
+    }
+  } else if (param->link == "probit") {
+    for (person = 0; person < n; person++) {
+      double r = R::pnorm(eta[person], 0, 1, 1, 0);
+      double phi = R::dnorm(eta[person], 0, 1, 0);
+      double dphi = -eta[person];
+      pi[person] = r;
+      d[person] = phi/(r*(1-r));
+      a[person] = phi*phi/(r*(1-r));
+      b[person] = (2*r-1)*phi/(r*(1-r)) + 2*dphi;
+    }
+  } else if (param->link == "cloglog") {
+    for (person = 0; person < n; person++) {
+      double r = 1 - exp(-exp(eta[person]));
+      double phi = exp(eta[person] - exp(eta[person]));
+      double dphi = 1 - exp(eta[person]);
+      pi[person] = r;
+      d[person] = phi/(r*(1-r));
+      a[person] = phi*phi/(r*(1-r));
+      b[person] = (2*r-1)*phi/(r*(1-r)) + 2*dphi;
     }
   }
 
   NumericMatrix imat(p,p); // X^T W X
-  for (i=0; i<p; i++) {
-    for (j=0; j<p; j++) {
-      for (person=0; person<n; person++) {
-        imat(i,j) += xw2(i,person)*xw2(j,person);
+  for (person = 0; person < n; person++) {
+    double f = param->freq[person];
+    double w = param->weight[person];
+    NumericVector z = param->z(person, _);
+    for (i=0; i<p; i++) {
+      for (j=0; j<=i; j++) {
+        imat(i,j) += f*w*a[person]*z[i]*z[j];
       }
     }
   }
 
+  for (i=0; i<p-1; i++) {
+    for (j=i+1; j<p; j++) {
+      imat(i,j) = imat(j,i);
+    }
+  }
+
   double toler = 1e-12;
   NumericMatrix var = invsympd(imat, p, toler);
 
-  // diagonals of the hat matrix H = W^{1/2} X (X^T W X)^{-1} X^T W^{1/2}
-  NumericVector Hdiag(n);
-  for (person=0; person<n; person++) {
+  NumericVector score(p);
+  for (person = 0; person < n; person++) {
+    double f = param->freq[person];
+    double w = param->weight[person];
+    NumericVector z = param->z(person, _);
+    double h = 0; // diagonal of H = W^{1/2}*X*inverse(X^T W X)*X^T W^{1/2}
     for (i=0; i<p; i++) {
       for (j=0; j<p; j++) {
-        Hdiag[person] += xw2(i,person)*var(i,j)*xw2(j,person);
+        h += var(i,j)*z[i]*z[j];
+      }
+    }
+    h *= f*w*a[person];
+
+    double resid = param->y[person] - pi[person];
+    double u = f*w*resid*d[person] + 0.5*b[person]*h;
+    for (i=0; i<p; i++) {
+      score[i] += u*z[i];
+    }
+  }
+
+  return score;
+}
+
+
+// Firth's bias-reducing score vector
+NumericVector f_firth_score_0(int p, NumericVector par, void *ex) {
+  logparams *param = (logparams *) ex;
+  int n = param->n;
+  int person, i, j, k, l;
+
+  NumericVector eta(n);
+  for (person = 0; person < n; person++) {
+    eta[person] = param->offset[person];
+    for (i=0; i<p; i++) {
+      eta[person] += par[i]*param->z(person,i);
+    }
+  }
+
+  NumericVector pi(n), d(n), a(n), b(n), c(n);
+  if (param->link == "logit") {
+    for (person = 0; person < n; person++) {
+      double r = R::plogis(eta[person], 0, 1, 1, 0);
+      pi[person] = r;
+      d[person] = 1;
+      a[person] = r*(1-r);
+      b[person] = 0;
+      c[person] = 1-2*r;
+    }
+  } else if (param->link == "probit") {
+    for (person = 0; person < n; person++) {
+      double r = R::pnorm(eta[person], 0, 1, 1, 0);
+      double phi = R::dnorm(eta[person], 0, 1, 0);
+      double dphi = -eta[person];
+      pi[person] = r;
+      d[person] = phi/(r*(1-r));
+      a[person] = phi*phi/(r*(1-r));
+      b[person] = (2*r-1)*phi/(r*(1-r)) + dphi;
+      c[person] = 2*(2*r-1)*phi/(r*(1-r)) + 3*dphi;
+    }
+  } else if (param->link == "cloglog") {
+    for (person = 0; person < n; person++) {
+      double r = 1 - exp(-exp(eta[person]));
+      double phi = exp(eta[person] - exp(eta[person]));
+      double dphi = 1 - exp(eta[person]);
+      pi[person] = r;
+      d[person] = phi/(r*(1-r));
+      a[person] = phi*phi/(r*(1-r));
+      b[person] = (2*r-1)*phi/(r*(1-r)) + dphi;
+      c[person] = 2*(2*r-1)*phi/(r*(1-r)) + 3*dphi;
+    }
+  }
+
+  // information matrix and the covariance matrix of the score vector
+  NumericMatrix imat(p,p), kmat(p,p);
+  for (person = 0; person < n; person++) {
+    double f = param->freq[person];
+    double w = param->weight[person];
+    NumericVector z = param->z(person, _);
+    for (i=0; i<p; i++) {
+      for (j=0; j<=i; j++) {
+        imat(i,j) += f*w*a[person]*z[i]*z[j];
+        kmat(i,j) += f*w*w*a[person]*z[i]*z[j];
       }
     }
   }
 
+  for (i=0; i<p-1; i++) {
+    for (j=i+1; j<p; j++) {
+      imat(i,j) = imat(j,i);
+      kmat(i,j) = kmat(j,i);
+    }
+  }
+
+  double toler = 1e-12;
+  NumericMatrix var = invsympd(imat, p, toler);
+
+  // sandwich variance of model parameters: inv(imat)*kmat*inv(imat)
+  NumericMatrix san(p,p);
+  for (i=0; i<p; i++) {
+    for (j=0; j<=i; j++) {
+      for (k=0; k<p; k++) {
+        for (l=0; l<p; l++) {
+          san(i,j) += var(i,k)*kmat(k,l)*var(l,j);
+        }
+      }
+    }
+  }
+
+  for (i=0; i<p-1; i++) {
+    for (j=i+1; j<p; j++) {
+      san(i,j) = san(j,i);
+    }
+  }
+
+
   NumericVector score(p);
   for (person = 0; person < n; person++) {
     double f = param->freq[person];
     double w = param->weight[person];
-    double v = param->y[person] - R::plogis(eta[person], 0, 1, 1, 0);
     NumericVector z = param->z(person, _);
+    double g = 0, h = 0;
     for (i=0; i<p; i++) {
-      score[i] += (f*w*v + Hdiag[person]*(0.5 - pi[person]))*z[i];
+      for (j=0; j<p; j++) {
+        g += var(i,j)*z[i]*z[j];
+        h += san(i,j)*z[i]*z[j];
+      }
+    }
+    g *= f*w*w*a[person];
+    h *= f*w*a[person];
+
+    double resid = param->y[person] - pi[person];
+    double u = f*w*resid*d[person] + (-g*b[person] + 0.5*h*c[person]);
+    for (i=0; i<p; i++) {
+      score[i] += u*z[i];
     }
   }
 
@@ -199,11 +420,36 @@ NumericMatrix f_ressco_0(int p, NumericVector par, void *ex) {
   }
 
   NumericMatrix resid(n, p);
-  for (person = 0; person < n; person++) {
-    double v = param->y[person] - R::plogis(eta[person], 0, 1, 1, 0);
-    NumericVector z = param->z(person, _);
-    for (i=0; i<p; i++) {
-      resid(person, i) = v*z[i];
+  if (param->link == "logit") {
+    for (person = 0; person < n; person++) {
+      double r = R::plogis(eta[person], 0, 1, 1, 0);
+      double v = param->y[person] - r;
+      NumericVector z = param->z(person, _);
+      for (i=0; i<p; i++) {
+        resid(person, i) = v*z[i];
+      }
+    }
+  } else if (param->link == "probit") {
+    for (person = 0; person < n; person++) {
+      double r = R::pnorm(eta[person], 0, 1, 1, 0);
+      double phi = R::dnorm(eta[person], 0, 1, 0);
+      double d = phi/(r*(1-r));
+      double v = param->y[person] - r;
+      NumericVector z = param->z(person, _);
+      for (i=0; i<p; i++) {
+        resid(person, i) = v*d*z[i];
+      }
+    }
+  } else if (param->link == "cloglog") {
+    for (person = 0; person < n; person++) {
+      double r = 1 - exp(-exp(eta[person]));
+      double phi = exp(eta[person] - exp(eta[person]));
+      double d = phi/(r*(1-r));
+      double v = param->y[person] - r;
+      NumericVector z = param->z(person, _);
+      for (i=0; i<p; i++) {
+        resid(person, i) = v*d*z[i];
+      }
     }
   }
 
@@ -212,8 +458,10 @@ NumericMatrix f_ressco_0(int p, NumericVector par, void *ex) {
 
 
 // underlying optimization algorithm for logisreg
+//   colfit: vector of indices of parameters to update
+//   ncolfit: number of parameters to update
 List logisregloop(int p, NumericVector par, void *ex,
-                  int maxiter, double eps, bool firth,
+                  int maxiter, double eps, bool firth, bool bc,
                   IntegerVector colfit, int ncolfit) {
   logparams *param = (logparams *) ex;
   int i, j, iter, halving = 0;
@@ -222,24 +470,30 @@ List logisregloop(int p, NumericVector par, void *ex,
   double toler = 1e-12;
   NumericVector beta(p), newbeta(p);
   double loglik, newlk = 0;
-  NumericVector u(p);
+  NumericVector u(p), oldu(p), newu(p);
   NumericMatrix imat(p,p);
   NumericVector u1(ncolfit);
   NumericMatrix imat1(ncolfit, ncolfit);
 
-  
+
   // initial beta and log likelihood
   for (i=0; i<p; i++) {
     beta[i] = par[i];
   }
-  
+
   if (!firth) {
     loglik = f_llik_0(p, beta, param);
     u = f_score_0(p, beta, param);
   } else {
     loglik = f_pen_llik_0(p, beta, param);
-    u = f_pen_score_0(p, beta, param);
+    if (bc) {
+      u = f_firth_score_0(p, beta, param);
+    } else {
+      u = f_pen_score_0(p, beta, param);
+    }
   }
+  if (firth && bc) oldu = clone(u);
+
   for (i=0; i<ncolfit; i++) {
     u1[i] = u[colfit[i]];
   }
@@ -268,6 +522,8 @@ List logisregloop(int p, NumericVector par, void *ex,
     if (!firth) newlk = f_llik_0(p, newbeta, param);
     else newlk = f_pen_llik_0(p, newbeta, param);
 
+    if (firth && bc) newu = f_firth_score_0(p, newbeta, param);
+
     // check convergence
     fail = std::isnan(newlk) || std::isinf(newlk) == 1;
 
@@ -275,7 +531,9 @@ List logisregloop(int p, NumericVector par, void *ex,
       break;
     }
 
-    if (fail || newlk < loglik) { // adjust step size if likelihood decreases
+    if (fail || (!(firth && bc) && newlk < loglik) ||
+        (firth && bc && sum(newu*newu) > sum(oldu*oldu))) {
+      // adjust step size if likelihood decreases
       halving++;
       for (i=0; i<p; i++) {
         newbeta[i] = (beta[i] + newbeta[i])/2;
@@ -288,8 +546,15 @@ List logisregloop(int p, NumericVector par, void *ex,
       }
       loglik = newlk;
 
-      if (!firth) u = f_score_0(p, beta, param);
-      else u = f_pen_score_0(p, beta, param);
+      if (!firth) {
+        u = f_score_0(p, beta, param);
+      } else if (bc) {
+        u = f_firth_score_0(p, beta, param);
+      } else {
+        u = f_pen_score_0(p, beta, param);
+      }
+      if (firth && bc) oldu = clone(u);
+
       for (i=0; i<ncolfit; i++) {
         u1[i] = u[colfit[i]];
       }
@@ -343,15 +608,18 @@ List logisregloop(int p, NumericVector par, void *ex,
 
 
 // confidence limit of profile likelihood method
+//   k: index of the parameter to obtain PL CI
+//   which: computation direction (-1 for lower limit or 1 for upper limit)
+//   l0: boundary of profile loglik = max loglik - 0.5*qchisq(1-alpha, 1)
+// refer to SAS PROC LOGISTIC documentation for Likelihood Ratio-Based
+// Confidence Intervals for Parameters
 double logisregplloop(int p, NumericVector par, void *ex,
-                      int maxiter, double eps, bool firth,
+                      int maxiter, double eps, bool firth, bool bc,
                       int k, int which, double l0) {
   logparams *param = (logparams *) ex;
 
   int i, j, iter;
-  bool fail;
-
-  NumericMatrix z1 = param->z;
+  bool fail = 0;
 
   NumericVector beta(p), newbeta(p);
   double loglik, newlk;
@@ -370,7 +638,11 @@ double logisregplloop(int p, NumericVector par, void *ex,
     u = f_score_0(p, beta, param);
   } else {
     loglik = f_pen_llik_0(p, beta, param);
-    u = f_pen_score_0(p, beta, param);
+    if (bc) {
+      u = f_firth_score_0(p, beta, param);
+    } else {
+      u = f_pen_score_0(p, beta, param);
+    }
   }
 
   imat = f_info_0(p, beta, param);
@@ -406,51 +678,57 @@ double logisregplloop(int p, NumericVector par, void *ex,
   for (iter=0; iter<maxiter; iter++) {
     if (!firth) newlk = f_llik_0(p, newbeta, param);
     else newlk = f_pen_llik_0(p, newbeta, param);
-    
+
     // check convergence
     fail = std::isnan(newlk) || std::isinf(newlk) == 1;
-    
-    if (!fail && fabs((newlk - l0)/l0) < eps) {
+
+    if (!fail && fabs(newlk - l0) < eps && w < eps) {
       break;
     }
-    
+
     for (i=0; i<p; i++) {
       beta[i] = newbeta[i];
     }
     loglik = newlk;
-    
-    if (!firth) u = f_score_0(p, beta, param);
-    else u = f_pen_score_0(p, beta, param);
-    
+
+    if (!firth) {
+      u = f_score_0(p, beta, param);
+    } else if (bc) {
+      u = f_firth_score_0(p, beta, param);
+    } else {
+      u = f_pen_score_0(p, beta, param);
+    }
+
     imat = f_info_0(p, beta, param);
-    
+
     // Lagrange multiplier method as used in SAS PROC LOGISTIC
     v = invsympd(imat, p, toler);
     v = -1.0*v;
-    
+
     w = 0;
     for (i=0; i<p; i++) {
       for (j=0; j<p; j++) {
         w += u[i]*v(i,j)*u[j];
       }
     }
+
     underroot = 2*(l0 - loglik + 0.5*w)/v(k,k);
     lambda = underroot < 0.0 ? 0.0 : which*sqrt(underroot);
     u[k] += lambda;
-    
+
     delta.fill(0.0);
     for (i=0; i<p; i++) {
       for (j=0; j<p; j++) {
         delta[i] -= v(i,j)*u[j];
       }
     }
-    
+
     // update beta
     for (i=0; i<p; i++) {
       newbeta[i] = beta[i] + delta[i];
     }
   }
-  
+
   if (iter == maxiter) fail = 1;
 
   if (fail) {
@@ -470,8 +748,10 @@ List logisregcpp(const DataFrame data,
                  const std::string weight = "",
                  const std::string offset = "",
                  const std::string id = "",
+                 const std::string link = "logit",
                  const bool robust = 0,
                  const bool firth = 0,
+                 const bool bc = 0,
                  const bool flic = 0,
                  const bool plci = 0,
                  const double alpha = 0.05) {
@@ -500,7 +780,6 @@ List logisregcpp(const DataFrame data,
   if (!has_event) {
     stop("data must contain the event variable");
   }
-
   NumericVector eventn(n);
   if (has_event) {
     NumericVector eventnz = data[event];
@@ -518,7 +797,6 @@ List logisregcpp(const DataFrame data,
   for (i=0; i<n; i++) {
     zn(i,0) = 1; // intercept
   }
-
   for (j=0; j<p-1; j++) {
     String zj = covariates[j];
     if (!hasVariable(data, zj)) {
@@ -531,7 +809,6 @@ List logisregcpp(const DataFrame data,
   }
 
   bool has_freq = hasVariable(data, freq);
-
   NumericVector freqn(n, 1.0);
   if (has_freq) {
     NumericVector freqnz = data[freq];
@@ -542,7 +819,6 @@ List logisregcpp(const DataFrame data,
   }
 
   bool has_weight = hasVariable(data, weight);
-
   NumericVector weightn(n, 1.0);
   if (has_weight) {
     NumericVector weightnz = data[weight];
@@ -553,7 +829,6 @@ List logisregcpp(const DataFrame data,
   }
 
   bool has_offset = hasVariable(data, offset);
-
   NumericVector offsetn(n);
   if (has_offset) {
     NumericVector offsetnz = data[offset];
@@ -561,7 +836,6 @@ List logisregcpp(const DataFrame data,
   }
 
   bool has_id = hasVariable(data, id);
-
   IntegerVector idn(n);
   if (!has_id) {
     idn = seq(1,n);
@@ -586,6 +860,13 @@ List logisregcpp(const DataFrame data,
     }
   }
 
+  std::string link1 = link;
+  std::for_each(link1.begin(), link1.end(), [](char & c) {
+    c = static_cast<char>(std::tolower(static_cast<unsigned char>(c)));
+  });
+  if (!(link1=="logit" || link1=="probit" || link1=="cloglog")) {
+    stop("Invalid value for link");
+  }
 
   // sort the data by rep
   IntegerVector order = seq(0, n-1);
@@ -600,12 +881,12 @@ List logisregcpp(const DataFrame data,
   offsetn = offsetn[order];
   idn = idn[order];
   zn = subset_matrix_by_row(zn, order);
-  
+
   // exclude observations with missing values
   LogicalVector sub(n,1);
   for (i=0; i<n; i++) {
-    if ((repn[i] == NA_INTEGER) || (eventn[i] == NA_INTEGER) || 
-        (std::isnan(freqn[i])) || (std::isnan(weightn[i])) || 
+    if ((repn[i] == NA_INTEGER) || (eventn[i] == NA_INTEGER) ||
+        (std::isnan(freqn[i])) || (std::isnan(weightn[i])) ||
         (std::isnan(offsetn[i])) || (idn[i] == NA_INTEGER)) {
       sub[i] = 0;
     }
@@ -613,7 +894,7 @@ List logisregcpp(const DataFrame data,
       if (std::isnan(zn(i,j+1))) sub[i] = 0;
     }
   }
-  
+
   order = which(sub);
   repn = repn[order];
   eventn = eventn[order];
@@ -622,7 +903,7 @@ List logisregcpp(const DataFrame data,
   offsetn = offsetn[order];
   idn = idn[order];
   zn = subset_matrix_by_row(zn, order);
-  n = sum(sub);
+  n = sum(sub); // number of nonmissing observations
 
   // identify the locations of the unique values of rep
   IntegerVector idx(1,0);
@@ -639,7 +920,7 @@ List logisregcpp(const DataFrame data,
   IntegerVector rep01 = seq(1,nreps);
   NumericVector nobs(nreps), nevents(nreps);
   NumericVector loglik0(nreps), loglik1(nreps);
-  NumericVector regloglik0(nreps), regloglik1(nreps);
+  NumericVector regloglik0(nreps), regloglik1(nreps); // regular loglik
   IntegerVector niter(nreps);
 
   IntegerVector rep0(nreps*p);
@@ -648,10 +929,10 @@ List logisregcpp(const DataFrame data,
   NumericMatrix vbeta0(nreps*p,p), rvbeta0(nreps*p,p);
   NumericVector lb0(nreps*p), ub0(nreps*p), prob0(nreps*p);
   StringVector clparm0(nreps*p);
-  
+
   NumericVector linear_predictors(n), fitted_values(n);
 
-  int N=0;
+  int N=0; // offset for the current replication data set
   for (h=0; h<nreps; h++) {
     IntegerVector q1 = Range(idx[h], idx[h+1]-1);
     int n1 = static_cast<int>(q1.size());
@@ -663,6 +944,7 @@ List logisregcpp(const DataFrame data,
     IntegerVector id1 = idn[q1];
     NumericMatrix z1 = subset_matrix_by_row(zn, q1);
 
+    // number of trials and number of events accounting for frequencies
     nobs[h] = sum(freq1);
     nevents[h] = sum(freq1*event1);
 
@@ -672,7 +954,6 @@ List logisregcpp(const DataFrame data,
       num += freq1[i]*weight1[i]*event1[i];
       den += freq1[i]*weight1[i];
     }
-
     if (firth) {
       num += 0.5;
       den += 1.0;
@@ -682,10 +963,16 @@ List logisregcpp(const DataFrame data,
     bint0[0] = R::qlogis(num/den, 0, 1, 1, 0);
 
     IntegerVector colfit0(1);
-    logparams param = {n1, event1, z1, freq1, weight1, offset1};
-    List outint = logisregloop(p, bint0, &param, 30, 1.0e-9, firth,
+    logparams param = {n1, link1, event1, z1, freq1, weight1, offset1};
+    List outint = logisregloop(p, bint0, &param, 100, 1.0e-9, firth, 0,
                                colfit0, 1);
 
+    if (firth && bc) {
+      NumericVector bint1 = outint["coef"];
+      outint = logisregloop(p, bint1, &param, 100, 1.0e-9, firth, bc,
+                            colfit0, 1);
+    }
+
     NumericVector bint = outint["coef"];
     NumericMatrix vbint = as<NumericMatrix>(outint["var"]);
 
@@ -696,7 +983,13 @@ List logisregcpp(const DataFrame data,
     if (p > 1) {
       // parameter estimates and standard errors for the full model
       IntegerVector colfit = seq(0,p-1);
-      out = logisregloop(p, bint, &param, 30, 1.0e-9, firth, colfit, p);
+      out = logisregloop(p, bint, &param, 100, 1.0e-9, firth, 0, colfit, p);
+
+      if (firth && bc) {
+        NumericVector bint1 = out["coef"];
+        out = logisregloop(p, bint1, &param, 100, 1.0e-9, firth, bc,
+                           colfit, p);
+      }
 
       bool fail = out["fail"];
       if (fail) stop("The algorithm in logisregr did not converge");
@@ -707,17 +1000,17 @@ List logisregcpp(const DataFrame data,
       // intercept correction
       if (flic) {
         NumericVector lp(n1);  // linear predictor excluding intercept
-        for (int person=0; person<n1; person++) {
+        for (int person = 0; person < n1; person++) {
           lp[person] = offset1[person];
           for (i=1; i<p; i++) {
             lp[person] += b[i]*z1(person,i);
           }
         }
-        
-        logparams param0 = {n1, event1, z1, freq1, weight1, lp};
-        
+
+        logparams param0 = {n1, link1, event1, z1, freq1, weight1, lp};
         NumericVector bint00(1, bint0[0]);
-        outint = logisregloop(1, bint00, &param0, 30, 1.0e-9, 0, colfit0, 1);
+        outint = logisregloop(1, bint00, &param0, 100, 1.0e-9, 0, 0,
+                              colfit0, 1);
         double a = as<double>(outint["coef"]);
         double va = as<double>(outint["var"]);
 
@@ -754,7 +1047,7 @@ List logisregcpp(const DataFrame data,
       out = outint;
 
       if (flic) {
-        out = logisregloop(p, bint0, &param, 30, 1.0e-9, 0, colfit0, 1);
+        out = logisregloop(p, bint0, &param, 100, 1.0e-9, 0, 0, colfit0, 1);
         b = out["coef"];
         vb = as<NumericMatrix>(outint["var"]);
       }
@@ -767,13 +1060,11 @@ List logisregcpp(const DataFrame data,
 
     for (i=0; i<p; i++) {
       rep0[h*p+i] = h+1;
-
       if (i==0) {
         par0[h*p+i] = "(Intercept)";
       } else {
         par0[h*p+i] = covariates[i-1];
       }
-
       beta0[h*p+i] = b[i];
       sebeta0[h*p+i] = seb[i];
       for (j=0; j<p; j++) {
@@ -781,30 +1072,33 @@ List logisregcpp(const DataFrame data,
       }
     }
 
-    
     // linear predictors and fitted values
     int person;
     NumericVector eta(n1);
-    for (person=0; person<n1; person++) {
+    for (person = 0; person < n1; person++) {
       eta[person] = offset1[person];
       for (i=0; i<p; i++) {
         eta[person] += b[i]*z1(person,i);
       }
-    }
-    
-    NumericVector pi(n1);
-    for (person=0; person<n1; person++) {
-      pi[person] = R::plogis(eta[person], 0, 1, 1, 0);
-    }
-    
-    for (person=0; person<n1; person++) {
       linear_predictors[N+person] = eta[person];
-      fitted_values[N+person] = pi[person];
     }
-    
+
+    if (link1 == "logit") {
+      for (person = 0; person < n1; person++) {
+        fitted_values[N+person] = R::plogis(eta[person], 0, 1, 1, 0);
+      }
+    } else if (link1 == "probit") {
+      for (person = 0; person < n1; person++) {
+        fitted_values[N+person] = R::pnorm(eta[person], 0, 1, 1, 0);
+      }
+    } else if (link1 == "cloglog") {
+      for (person = 0; person < n1; person++) {
+        fitted_values[N+person] = 1 - exp(-exp(eta[person]));
+      }
+    }
+
     N += n1;
-    
-    
+
     niter[h] = out["iter"];
 
     // robust variance estimates
@@ -911,8 +1205,10 @@ List logisregcpp(const DataFrame data,
 
       if (!(firth && flic)) {
         for (k=0; k<p; k++) {
-          lb[k] = logisregplloop(p, b, &param, 30, 1.0e-8, firth, k, -1, l0);
-          ub[k] = logisregplloop(p, b, &param, 30, 1.0e-8, firth, k, 1, l0);
+          lb[k] = logisregplloop(p, b, &param, 100, 1.0e-6, firth, bc,
+                                 k, -1, l0);
+          ub[k] = logisregplloop(p, b, &param, 100, 1.0e-6, firth, bc,
+                                 k, 1, l0);
 
           IntegerVector colfit1(p-1);
           for (i=0; i<k; i++) {
@@ -923,13 +1219,18 @@ List logisregcpp(const DataFrame data,
           }
 
           NumericVector b0(p);
-          List out0 = logisregloop(p, b0, &param, 30, 1.0e-8, firth,
+          List out0 = logisregloop(p, b0, &param, 100, 1.0e-8, firth, 0,
                                    colfit1, p-1);
+          if (firth && bc) {
+            NumericVector b1 = out0["coef"];
+            out0 = logisregloop(p, b1, &param, 100, 1.0e-8, firth, bc,
+                                colfit1, p-1);
+          }
           double lmax0 = out0["loglik"];
           prob[k] = R::pchisq(-2*(lmax0 - lmax), 1, 0, 0);
           clparm[k] = "PL";
         }
-      } else {
+      } else { // Wald CI for intercept and PL CI for slopes
         if (!robust) {
           lb[0] = b[0] - zcrit*seb[0];
           ub[0] = b[0] + zcrit*seb[0];
@@ -942,8 +1243,10 @@ List logisregcpp(const DataFrame data,
         clparm[0] = "Wald";
 
         for (k=1; k<p; k++) {
-          lb[k] = logisregplloop(p, b, &param, 30, 1.0e-8, firth, k, -1, l0);
-          ub[k] = logisregplloop(p, b, &param, 30, 1.0e-8, firth, k, 1, l0);
+          lb[k] = logisregplloop(p, b, &param, 100, 1.0e-6, firth, bc,
+                                 k, -1, l0);
+          ub[k] = logisregplloop(p, b, &param, 100, 1.0e-6, firth, bc,
+                                 k, 1, l0);
 
           IntegerVector colfit1(p-1);
           for (i=0; i<k; i++) {
@@ -954,8 +1257,13 @@ List logisregcpp(const DataFrame data,
           }
 
           NumericVector b0(p);
-          List out0 = logisregloop(p, b0, &param, 30, 1.0e-8, firth,
+          List out0 = logisregloop(p, b0, &param, 100, 1.0e-8, firth, 0,
                                    colfit1, p-1);
+          if (firth && bc) {
+            NumericVector b1 = out0["coef"];
+            out0 = logisregloop(p, b1, &param, 100, 1.0e-8, firth, bc,
+                                colfit1, p-1);
+          }
           double lmax0 = out0["loglik"];
           prob[k] = R::pchisq(-2*(lmax0 - lmax), 1, 0, 0);
           clparm[k] = "PL";
@@ -987,7 +1295,6 @@ List logisregcpp(const DataFrame data,
     if (firth) {
       loglik0[h] = f_pen_llik_0(p, bint, &param);
       loglik1[h] = f_pen_llik_0(p, b, &param);
-
       regloglik0[h] = f_llik_0(p, bint, &param);
       regloglik1[h] = f_llik_0(p, b, &param);
     } else {
@@ -1008,8 +1315,10 @@ List logisregcpp(const DataFrame data,
     _["loglik1"] = loglik1,
     _["niter"] = niter,
     _["p"] = p,
+    _["link"] = link1,
     _["robust"] = robust,
     _["firth"] = firth,
+    _["bc"] = bc,
     _["flic"] = flic);
 
   if (firth) {
@@ -1046,12 +1355,10 @@ List logisregcpp(const DataFrame data,
       _["vbeta_naive"] = vbeta0);
   }
 
-  
   DataFrame fitted = DataFrame::create(
     Named("linear_predictors") = linear_predictors,
     Named("fitted_values") = fitted_values);
-  
-  
+
   if (has_rep) {
     for (i=0; i<p_rep; i++) {
       String s = rep[i];
@@ -1073,7 +1380,6 @@ List logisregcpp(const DataFrame data,
       }
     }
   }
-  
 
   List result = List::create(
     _["sumstat"] = sumstat,
